@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-tray-extension.c,v 1.13 2005/02/13 21:43:42 chpe Exp $
+ *  $Id: ephy-bookmarks-tray-extension.c,v 1.14 2005/03/03 14:42:41 jframeau Exp $
  */
 
 #include "config.h"
@@ -32,7 +32,7 @@
 #include <epiphany/ephy-extension.h>
 #include "ephy-bookmarks-menu.h"
 #include "ephy-gui.h"
-#include "ephy-link.h"
+#include <epiphany/ephy-link.h>
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
