@@ -15,12 +15,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ErrorViewerURICheckerObserver.cpp,v 1.7 2004/11/15 15:07:37 chpe Exp $
+ *  $Id: ErrorViewerURICheckerObserver.cpp,v 1.8 2004/12/08 15:33:52 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
+#include "mozilla-config.h"
+
 #include "config.h"
-#endif
 
 #include "link-checker.h"
 
