@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-extensions-manager-ui-extension.c,v 1.3 2005/01/13 16:15:48 chpe Exp $
+ *  $Id: ephy-extensions-manager-ui-extension.c,v 1.4 2005/01/28 22:29:41 chpe Exp $
  */
 
 #include "config.h"
@@ -116,7 +116,7 @@ ephy_extensions_manager_ui_extension_register_type (GTypeModule *module)
 static void
 ephy_extensions_manager_ui_extension_init (EphyExtensionsManagerUIExtension *extension)
 {
-	LOG ("EphyExtensionsManagerUIExtension initialising")
+	LOG ("EphyExtensionsManagerUIExtension initialising");
 
 	extension->priv = EPHY_EXTENSIONS_MANAGER_UI_EXTENSION_GET_PRIVATE (extension);
 
@@ -128,7 +128,7 @@ ephy_extensions_manager_ui_extension_finalize (GObject *object)
 {
 	EphyExtensionsManagerUIExtension *extension = EPHY_EXTENSIONS_MANAGER_UI_EXTENSION (object);
 
-	LOG ("EphyExtensionsManagerUIExtension finalising")
+	LOG ("EphyExtensionsManagerUIExtension finalising");
 
 	if (extension->priv->dialog != NULL)
 	{
@@ -201,7 +201,7 @@ impl_attach_window (EphyExtension *ext,
 			       "ExtensionsManagerUISep2", NULL,
 			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 
-	LOG ("EphyExtensionsManagerUIExtension attach_window")
+	LOG ("EphyExtensionsManagerUIExtension attach_window");
 }
 
 static void
@@ -222,7 +222,7 @@ impl_detach_window (EphyExtension *ext,
 
 	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
 
-	LOG ("EphyExtensionsManagerUIExtension detach_window")
+	LOG ("EphyExtensionsManagerUIExtension detach_window");
 }
 
 static void
