@@ -18,12 +18,10 @@
  *
  *  This file is based on work of Daniel Erat for galeon 1.
  *
- *  $Id: ephy-gesture.c,v 1.4 2004/12/05 22:43:59 chpe Exp $
+ *  $Id: ephy-gesture.c,v 1.6 2005/01/28 22:30:04 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-gesture.h"
 #include "ephy-debug.h"
@@ -239,7 +237,7 @@ ephy_gesture_finalize (GObject *object)
 	g_object_unref (gesture->priv->window);
 	g_object_unref (gesture->priv->event);
 
-	LOG ("EphyGesture finalised %p", object)
+	LOG ("EphyGesture finalised %p", object);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
