@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2004 Adam Hooper
- *  Copyright (C) 2004 Christian Persch
+ *  Copyright (C) 2004, 2005 Christian Persch
+ *  Copyright (C) 2004, 2005 Jean-François Rameau
  *
  *  Heavily based on Galeon's:
  *      Copyright (C) 2003 Philip Langdale
@@ -22,7 +23,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: page-info-dialog.c,v 1.39 2005/01/28 22:30:13 chpe Exp $
+ *  $Id: page-info-dialog.c,v 1.43 2005/03/03 10:59:12 chpe Exp $
  */
 
 #include "config.h"
@@ -66,6 +67,11 @@
 #include <time.h>
 #include <string.h>
 
+/* Forward declarations */
+static void page_info_set_text (PageInfoDialog *dialog,
+		    		const char *prop,
+		    		const char *text);
+
 /* Glade callbacks */
 void page_info_dialog_close_button_clicked_cb	  (GtkWidget *button,
 						   PageInfoDialog *dialog);
@@ -79,6 +85,7 @@ enum {
 	MEDIA_PAGE,
 	LINKS_PAGE,
 	FORMS_PAGE,
+	METADATA_PAGE,
 	/*
 	PAGE_INFO_MEDIA,
 	*/
@@ -137,10 +144,6 @@ enum
 	PROP_GENERAL_REFERRING_URL,
 	PROP_GENERAL_MODIFIED,
 	PROP_GENERAL_EXPIRES,
-/*
-	PROP_GENERAL_META_TREEVIEW,
-*/
-	PROP_FORMS_FORM_TREEVIEW,
 
 	PROP_LINKS_LINK_TREEVIEW,
 
@@ -149,6 +152,16 @@ enum
 	PROP_MEDIA_MEDIUM_VPANED,
 	PROP_MEDIA_SAVE_BUTTON,
 
+	PROP_FORMS_FORM_TREEVIEW,
+
+	PROP_META_DC_BOX,
+	PROP_META_DC_TITLE,
+	PROP_META_DC_DESC,
+	PROP_META_DC_DATE,
+	PROP_META_DC_FORMAT,
+	PROP_META_OTHER_LABEL,
+	PROP_META_TREEVIEW,
+	
 /*
 	PROP_SECURITY_CERT_TITLE,
 	PROP_SECURITY_CERT_INFO,
@@ -175,9 +188,6 @@ EphyDialogProperty properties [] =
 	{ "page_info_referring_url",	NULL, PT_NORMAL, 0 },
 	{ "page_info_modified",		NULL, PT_NORMAL, 0 },
 	{ "page_info_expires",		NULL, PT_NORMAL, 0 },
-	/*{ "page_info_meta_list",	NULL, PT_NORMAL, 0 },*/
-
-	{ "page_info_form_list",	NULL, PT_NORMAL, 0 },
 
 	{ "page_info_link_list",	NULL, PT_NORMAL, 0 },
 
@@ -186,6 +196,16 @@ EphyDialogProperty properties [] =
 	{ "page_info_media_vpaned",	NULL, PT_NORMAL, 0 },
 	{ "page_info_media_save",	NULL, PT_NORMAL, 0 },
 
+	{ "page_info_form_list",	NULL, PT_NORMAL, 0 },
+
+	{ "page_meta_dc_box",		NULL, PT_NORMAL, 0 },
+	{ "page_meta_dc_title",		NULL, PT_NORMAL, 0 },
+	{ "page_meta_dc_desc",		NULL, PT_NORMAL, 0 },
+	{ "page_meta_dc_date",		NULL, PT_NORMAL, 0 },
+	{ "page_meta_dc_format",	NULL, PT_NORMAL, 0 },
+	{ "page_meta_other_label",	NULL, PT_NORMAL, 0 },
+	{ "page_meta_list",		NULL, PT_NORMAL, 0 },
+
 	/*
 	{ PROP_SECURITY_CERT_TITLE,     "page_info_security_title", NULL, PT_NORMAL, NULL },
 	{ PROP_SECURITY_CERT_INFO,      "page_info_security_info",  NULL, PT_NORMAL, NULL },
@@ -199,18 +219,11 @@ EphyDialogProperty properties [] =
 
 static GtkTargetEntry drag_targets[] =
 {
-	{ EPHY_DND_URL_TYPE, 0, 0 }
+	{ EPHY_DND_URL_TYPE,	  0, 0 }//,
+	//{ EPHY_DND_URI_LIST_TYPE, 0, 1}
 };
 static int n_drag_targets = G_N_ELEMENTS (drag_targets);
 
-/*
-enum
-{
-	COL_META_NAME,
-	COL_META_CONTENT
-};
-*/
-
 static void page_info_dialog_class_init	(PageInfoDialogClass *klass);
 static void page_info_dialog_init	(PageInfoDialog *dialog);
 
@@ -248,66 +261,6 @@ page_info_dialog_register_type (GTypeModule *module)
 	return type;
 }
 
-/* not-yet ported stuff */
-
-/*
-static GtkTreeView *
-setup_meta_treeview (PageInfoDialog *dialog)
-{
-	GtkTreeView *treeview;
-	GtkListStore *liststore;
-	GtkCellRenderer *renderer;
-	GtkTreeViewColumn *column;
-	GtkTreeSelection *selection;
-
-	treeview = GTK_TREE_VIEW(galeon_dialog_get_control 
-				 (GALEON_DIALOG(dialog),
-				 PROP_GENERAL_META_TREEVIEW));
-	
-*/
-	/* set tree model */
-/*
-	liststore = gtk_list_store_new (2,
-					G_TYPE_STRING,
-					G_TYPE_STRING);
-	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
-	g_object_unref (liststore);
-
-	gtk_tree_view_set_headers_visible (treeview, TRUE);
-	selection = gtk_tree_view_get_selection (treeview);
-	gtk_tree_selection_set_mode (selection,
-				     GTK_SELECTION_SINGLE);
-	
-	renderer = gtk_cell_renderer_text_new ();
-
-	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_META_NAME,
-						     _("Name"),
-						     renderer,
-						     "text", COL_META_NAME,
-						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_META_NAME);
-	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_view_column_set_reorderable (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_META_NAME);
-
-	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_META_CONTENT, 
-						     _("Content"),
-						     renderer,
-						     "text", COL_META_CONTENT,
-						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_META_CONTENT);
-	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_view_column_set_reorderable (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_META_CONTENT);
-
-	return treeview;
-}
-*/
-
 /*
 static void
 setup_page_security (PageInfoDialog *dialog, EmbedPageProperties *props)
@@ -429,25 +382,6 @@ setup_page_security (PageInfoDialog *dialog, EmbedPageProperties *props)
 }
 */
 
-/*
-static void
-setup_page_general_add_meta_tag(GtkTreeView *treeView, EmbedPageMetaTag *tag)
-{
-	GtkListStore *store;
-	GtkTreeIter iter;
-
-	store = GTK_LIST_STORE(gtk_tree_view_get_model
-			       (GTK_TREE_VIEW(treeView)));
-
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store,
-			   &iter,
-			   COL_META_NAME, tag->name,
-			   COL_META_CONTENT, tag->content,
-			    -1);
-}
-*/
-
 /*
 void
 page_info_dialog_set_current_page (PageInfoDialog *dialog, PageInfoDialogPage page)
@@ -486,6 +420,29 @@ page_info_dialog_view_cert_button_clicked_cb (GtkWidget *button,
 	*/
 }
 
+/* Helper functions */
+
+static void
+page_info_set_text (PageInfoDialog *dialog,
+		    const char *prop,
+		    const char *text)
+{
+	GtkWidget *widget;
+
+	widget = ephy_dialog_get_control (EPHY_DIALOG (dialog), prop);
+
+	gtk_label_set_text (GTK_LABEL (widget), text ? text : "");
+}
+
+/* Set dnd cursor to the default dnd Gtk one and not the GtktreeView one */
+static void 
+page_info_drag_begin_cb (GtkWidget *widget, 
+		         GdkDragContext *dc, 
+		         gpointer user_data)
+{
+	gtk_drag_set_icon_default (dc);
+}
+
 /* a generic treeview info page */
 
 typedef struct _TreeviewInfoPage TreeviewInfoPage;
@@ -499,12 +456,96 @@ struct _TreeviewInfoPage
 	GtkListStore *store;
 	GtkTreeSelection *selection;
 	GtkTreeView *treeview;
+	TreeviewInfoPageFilterFunc filter_func;
 	GtkActionEntry *action_entries;
 	guint n_action_entries;
 	const char *popup_path;
-        TreeviewInfoPageFilterFunc filter_func;
+	const char *copy_action;
+	int copy_data_col;
 };
 
+/* Return a list of selected data rows (char *) */
+static GList *
+treeview_info_page_get_selected_rows (TreeviewInfoPage *page)
+{
+	GtkTreeModel *model;
+	char *data = NULL;
+	GList *rows, *l, *ret = NULL;
+
+	rows = gtk_tree_selection_get_selected_rows (page->selection, &model);
+
+	for (l = rows; l != NULL; l = l->next)
+	{
+		GtkTreePath *path = (GtkTreePath *) l->data;
+		GtkTreeIter iter;
+
+		if (gtk_tree_model_get_iter (model, &iter, path))
+		{
+			gtk_tree_model_get (model, &iter,
+					    page->copy_data_col, &data,
+					    -1);
+
+			ret = g_list_prepend (ret, data);
+		}
+	}
+
+	g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (rows);
+
+	return g_list_reverse (ret);
+}
+
+static char *
+treeview_info_page_get_selected_data (TreeviewInfoPage *page)
+{
+	GList *rows;
+	char *data;
+
+	g_return_val_if_fail (gtk_tree_selection_count_selected_rows (page->selection) == 1, NULL);
+
+	rows = treeview_info_page_get_selected_rows (page);
+	g_return_val_if_fail (rows != NULL, NULL);
+
+	data = (char *) rows->data;
+	g_list_free (rows);
+
+	return data;
+}
+
+static void
+treeview_info_page_copy_selected (GtkAction *action,
+				  TreeviewInfoPage *page)
+{
+	char *data;
+
+	data = treeview_info_page_get_selected_data (page);
+	if (data == NULL) return;
+		
+	gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
+				data, -1);
+	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+				data, -1);
+
+	g_free (data);
+}				  
+
+static void
+treeview_info_page_filter (TreeviewInfoPage *page)
+{
+	InfoPage *ipage = (InfoPage *) page;
+	PageInfoDialog *dialog = ipage->dialog;
+	GtkTreeSelection *selection = page->selection;
+	GtkAction *action;
+	int count;
+
+	action = gtk_action_group_get_action (dialog->priv->action_group,
+					      page->copy_action);
+	g_return_if_fail (action != NULL);
+
+	count = gtk_tree_selection_count_selected_rows (selection);
+	g_object_set (G_OBJECT (action), "sensitive", count == 1, NULL);
+}
+
 static gboolean
 treeview_info_page_show_popup (TreeviewInfoPage *page)
 {
@@ -599,25 +640,12 @@ treeview_info_page_construct (InfoPage *ipage)
 
 /* "General" page */
 
-static void
-page_info_set_text (PageInfoDialog *dialog,
-		    const char *prop,
-		    const char *text)
-{
-	GtkWidget *widget;
-	/* FIXME: Switch to prop strings instead of enum */
-	widget = ephy_dialog_get_control (EPHY_DIALOG (dialog), prop);
-
-	gtk_label_set_text (GTK_LABEL (widget), text ? text : "");
-}
-
 static void
 general_info_page_fill (InfoPage *page)
 {
 	PageInfoDialog *dialog = page->dialog;
 	EphyEmbed *embed = dialog->priv->embed;
 	EmbedPageProperties *props;
-	/* GtkTreeView *page_info_meta_list; */
 	const char *text;
 	const char *date_hack = "%c"; /* quiet gcc */
 	char date[128];
@@ -655,6 +683,7 @@ general_info_page_fill (InfoPage *page)
 		case EMBED_RENDER_QUIRKS:
 			text = _("Compatibility");
 			break;
+		case EMBED_RENDER_UNKNOWN:
 		default:
 			text = _("Undetermined");
 			break;
@@ -700,7 +729,7 @@ general_info_page_fill (InfoPage *page)
 	if (props->modification_time)
 	{
 		t = props->modification_time;
-		strftime (date, sizeof(date), date_hack, localtime_r (&t, &tm));
+		strftime (date, sizeof (date), date_hack, localtime_r (&t, &tm));
 		val = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
 		
 		page_info_set_text (dialog, properties[PROP_GENERAL_MODIFIED].id, val);
@@ -728,15 +757,6 @@ general_info_page_fill (InfoPage *page)
 				    _("Not specified"));
 
 	}
-
-	/*
-	page_info_meta_list = setup_meta_treeview(dialog);
-	for (i=props->metatags ; i ; i = i->next)
-	{
-		setup_page_general_add_meta_tag(page_info_meta_list,
-						(EmbedPageMetaTag*)i->data);
-	}	
-	*/
 }
 
 static InfoPage *
@@ -750,15 +770,6 @@ general_info_page_new (PageInfoDialog *dialog)
 	return ipage;
 }
 
-/* Set dnd cursor to the default dnd Gtk one and not the GtktreeView one */
-static void 
-page_info_drag_begin_cb(GtkWidget *widget, 
-		        GdkDragContext *dc, 
-		        gpointer dummy)
-{
-	gtk_drag_set_icon_default (dc);
-}
-
 /* "Media" page */
 
 #define MEDIUM_PANED_POSITION_DEFAULT	250
@@ -835,7 +846,7 @@ media_get_selected_medium_type (GtkTreeSelection *selection)
 
 	selected_row = gtk_tree_selection_get_selected_rows (selection, &model);
 
-	if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *)(selected_row->data)))
+	if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) selected_row->data))
 	{
 		gtk_tree_model_get (model, &iter, COL_MEDIUM_TYPE, &type, -1);
 	}
@@ -850,75 +861,47 @@ media_is_embedded_medium (EmbedPageMediumType type)
 	return type == MEDIUM_OBJECT || type == MEDIUM_EMBED;
 }
 
-/* Return a list of selected addresses (char *) */
-static GList *
-media_get_selected_medium_url (MediaInfoPage *page)
-{
-	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
-	GtkTreeModel *model;
-	char *address = NULL;
-	GList *selected_rows;
-	GList *ret = NULL, *l;
-
-	selected_rows = gtk_tree_selection_get_selected_rows (tpage->selection, &model);
-
-	for (l=selected_rows; l != NULL; l = l->next)
-	{
-		GtkTreeIter iter;
-
-		if (gtk_tree_model_get_iter (model, &iter, 
-					     (GtkTreePath *)(l->data)))
-		{
-			gtk_tree_model_get (model, &iter, COL_MEDIUM_URL, &address, -1);
-
-			ret = g_list_prepend (ret, address);
-		}
-	}
-
-	g_list_free (selected_rows);
-
-	return ret;
-}
-
 /* Save a medium given the destination directory */
 static void
-save_a_medium (const char *source, const char *dir)
+save_a_medium (const char *source,
+	       const char *dir)
 {
 	GnomeVFSURI *uri;
 	EphyEmbedPersist *persist;
 
 	uri = gnome_vfs_uri_new (source);
-
-	if (uri) 
+	if (uri != NULL)
 	{	
 		char *file_name;
 
 		file_name = gnome_vfs_uri_extract_short_name (uri);
-
 		if (file_name != NULL)
 		{
 			char *dest;
 
 			/* Persist needs a full path */
-			dest = g_strconcat (dir, "/", file_name, NULL);
+			dest = g_build_filename (dir, file_name, NULL);
 
 			persist = EPHY_EMBED_PERSIST (
 					ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 			ephy_embed_persist_set_source (persist, source);
-			ephy_embed_persist_set_dest   (persist, dest);
+			ephy_embed_persist_set_dest (persist, dest);
 			ephy_embed_persist_save (persist);
 
 			g_object_unref (persist);
-			gnome_vfs_uri_unref (uri);
 			g_free(dest);
 		}
-		g_free(file_name);
+
+		g_free (file_name);
+		gnome_vfs_uri_unref (uri);
 	}
 }
 
 /* Download at the same time all the selelected medium (rows) */
 static void
-download_path_response_cb (GtkDialog *fc, gint response, GList *rows)
+download_path_response_cb (GtkDialog *fc,
+			   int response,
+			   GList *rows)
 {
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
@@ -950,20 +933,20 @@ media_open_medium_cb (GtkAction *action,
 	/* FIXME: write me! :) */
 }
 
-
 static void
 media_save_medium_cb (gpointer ptr,
 		      MediaInfoPage *page)
 {
 	InfoPage *ipage = (InfoPage *) page;
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	PageInfoDialog *dialog = ipage->dialog;
 	EphyEmbedPersist *persist;
 	GList *rows;
 
-	rows = media_get_selected_medium_url (page);
+	rows = treeview_info_page_get_selected_rows (tpage);
 	if (g_list_length (rows) == 1)
 	{
-		char *address = (gchar *)(rows->data); 
+		char *address = (char *) rows->data;
 
 		if (address != NULL)
 		{
@@ -982,7 +965,7 @@ media_save_medium_cb (gpointer ptr,
 		g_list_foreach (rows, (GFunc) g_free, NULL);
 		g_list_free (rows);
 	}
-	else
+	else if (rows != NULL)
 	{
 		EphyFileChooser *fc;
 		GtkWidget *parent;
@@ -1010,6 +993,9 @@ background_download_completed_cb (EphyEmbedPersist *persist)
 {
 	const char *bg;
 	char *type;
+	guint32 user_time;
+
+	user_time = ephy_embed_persist_get_user_time (persist);
 
 	bg = ephy_embed_persist_get_dest (persist);
 	eel_gconf_set_string (CONF_DESKTOP_BG_PICTURE, bg);
@@ -1020,25 +1006,27 @@ background_download_completed_cb (EphyEmbedPersist *persist)
 		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE, "wallpaper");
 	}
 	g_free (type);
+
+        /* open the "Background Properties" capplet */
+        ephy_file_launch_desktop_file ("background.desktop", user_time);
 }
 
 static void
 media_set_image_as_background_cb (GtkAction *action,
 				  MediaInfoPage *page)
 {
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	EphyEmbedPersist *persist;
 	char *address;
 	char *dest, *base, *base_converted;
-	GList *rows;
 
-	rows = media_get_selected_medium_url (page);
-	g_return_if_fail (g_list_length (rows) == 1);
+	address = treeview_info_page_get_selected_data (tpage);
+	if (address == NULL) return;
 
-	address = (gchar *)(rows->data); 
-	if (address != NULL)
-	{
 	base = g_path_get_basename (address);
 	base_converted = g_filename_from_utf8 (base, -1, NULL, NULL, NULL);
+	if (base_converted == NULL) return;
+
 	dest = g_build_filename (ephy_dot_dir (), base_converted, NULL);
 
 	persist = EPHY_EMBED_PERSIST
@@ -1058,32 +1046,6 @@ media_set_image_as_background_cb (GtkAction *action,
 	g_free (dest);
 	g_free (base);
 	g_free (base_converted);
-	}
-	g_list_foreach (rows, (GFunc) g_free, NULL);
-	g_list_free (rows);
-}
-
-static void
-media_copy_medium_address_cb (GtkAction *action,
-			      MediaInfoPage *page)
-{
-	char *address;
-	GList *rows;
-
-	rows = media_get_selected_medium_url (page);
-	g_return_if_fail (g_list_length (rows) == 1);
-
-	address = (gchar *)(rows->data);
-	g_return_if_fail (address != NULL);
-	if (address != NULL)
-	{
-	gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
-				address, -1);
-	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-				address, -1);
-	}
-	g_list_foreach (rows, (GFunc) g_free, NULL);
-	g_list_free (rows);
 }
 
 static GtkActionEntry media_action_entries[] =
@@ -1099,7 +1061,7 @@ static GtkActionEntry media_action_entries[] =
 	  N_("_Copy Link Address"),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (media_copy_medium_address_cb) },
+	  G_CALLBACK (treeview_info_page_copy_selected) },
 	{ "SetAsBackground",
 	  NULL,
 	  N_("_Use as Background"),
@@ -1219,25 +1181,17 @@ media_drag_data_get_cb (GtkWidget *widget,
                         guint32 time,
                         MediaInfoPage *page)
 {
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	char *address;
-	GList *rows;
 
-	g_assert (widget != NULL);
-	g_return_if_fail (context != NULL);
-
-	rows = media_get_selected_medium_url (page);
-	g_return_if_fail (rows != NULL);
-
-	address = (gchar *)(rows->data);
-	g_return_if_fail (address != NULL);
+	address = treeview_info_page_get_selected_data (tpage);
+	if (address == NULL) return;
 
 	gtk_selection_data_set (selection_data,
 			selection_data->target,
 			8,
 			address,
 			strlen(address));
-	g_list_foreach (rows, (GFunc) g_free, NULL);
-	g_list_free (rows);
 }
 
 static void
@@ -1427,6 +1381,8 @@ media_info_page_new (PageInfoDialog *dialog)
 	ipage->fill = media_info_page_fill;
 
 	tpage->popup_path = "/MediaPopup";
+	tpage->copy_action = "CopyMediumAddress";
+	tpage->copy_data_col = COL_MEDIUM_URL;
 	tpage->action_entries = media_action_entries;
 	tpage->n_action_entries = G_N_ELEMENTS (media_action_entries);
 	tpage->filter_func = media_info_page_filter;
@@ -1450,41 +1406,6 @@ enum
 	COL_LINK_REL
 };
 
-static char *
-links_get_selected_link_url (LinksInfoPage *page)
-{
-	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	char *address = NULL;
-
-	if (gtk_tree_selection_get_selected (tpage->selection, &model, &iter))
-	{
-		gtk_tree_model_get (model, &iter,
-				    COL_LINK_URL, &address,
-				    -1);
-	}
-
-	return address;
-}
-
-static void
-links_copy_link_address_cb (GtkAction *action,
-			    LinksInfoPage *page)
-{
-	char *address;
-
-	address = links_get_selected_link_url (page);
-	if (address == NULL) return;
-
-	gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
-				address, -1);
-	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-				address, -1);
-
-	g_free (address);
-}
-
 static GtkActionEntry links_action_entries[] =
 {
 	{ "CopyLinkAddress", 
@@ -1492,7 +1413,7 @@ static GtkActionEntry links_action_entries[] =
 	  N_("_Copy Link Address"),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (links_copy_link_address_cb) },
+	  G_CALLBACK (treeview_info_page_copy_selected) },
 };
 
 static void
@@ -1503,12 +1424,10 @@ links_drag_data_get_cb (GtkWidget *widget,
                         guint32 time,
                         LinksInfoPage *page)
 {
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	char *address;
 
-	g_assert (widget != NULL);
-	g_return_if_fail (context != NULL);
-
-	address = links_get_selected_link_url (page);
+	address = treeview_info_page_get_selected_data (tpage);
 	if (address == NULL) return;
 
 	gtk_selection_data_set (selection_data,
@@ -1640,8 +1559,11 @@ links_info_page_new (PageInfoDialog *dialog)
 	ipage->fill = links_info_page_fill;
 
 	tpage->popup_path = "/LinksPopup";
+	tpage->copy_action = "CopyLinkAddress";
+	tpage->copy_data_col = COL_LINK_URL;
 	tpage->action_entries = links_action_entries;
 	tpage->n_action_entries = G_N_ELEMENTS (links_action_entries);
+	tpage->filter_func = treeview_info_page_filter;
 
 	return ipage;
 }
@@ -1662,16 +1584,17 @@ enum
 	COL_FORM_ACTION
 };
 
+/*
 static GtkActionEntry forms_action_entries[] =
 {
 };
+*/
 
 static void
 forms_info_page_construct (InfoPage *ipage)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
 	PageInfoDialog *dialog = ipage->dialog;
-
 	GtkTreeView *treeview;
 	GtkListStore *liststore;
 	GtkCellRenderer *renderer;
@@ -1772,8 +1695,200 @@ forms_info_page_new (PageInfoDialog *dialog)
 	ipage->fill = forms_info_page_fill;
 
 	tpage->popup_path = "/FormsPopup";
+/*
 	tpage->action_entries = forms_action_entries;
 	tpage->n_action_entries = G_N_ELEMENTS (forms_action_entries);
+*/
+	return ipage;
+}
+
+/* "Metadata" page */
+
+typedef struct _MetadataInfoPage MetadataInfoPage;
+
+struct _MetadataInfoPage
+{
+	TreeviewInfoPage tpage;
+};
+
+enum
+{
+	COL_META_NAME,
+	COL_META_CONTENT
+};
+
+static GtkActionEntry metadata_action_entries[] =
+{
+	{ "CopyMetadata", 
+	  GTK_STOCK_COPY,
+	  N_("_Copy"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (treeview_info_page_copy_selected) },
+};
+
+
+static void
+metadata_info_page_construct (InfoPage *ipage)
+{
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
+	PageInfoDialog *dialog = ipage->dialog;
+	GtkTreeView *treeview;
+	GtkListStore *liststore;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+
+	treeview = GTK_TREE_VIEW (ephy_dialog_get_control
+		(EPHY_DIALOG (dialog), properties[PROP_META_TREEVIEW].id));
+
+	liststore = gtk_list_store_new (2,
+					G_TYPE_STRING,
+					G_TYPE_STRING);
+	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
+	g_object_unref (liststore);
+
+	gtk_tree_view_set_headers_visible (treeview, TRUE);
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     COL_META_NAME,
+						     _("Name"),
+						     renderer,
+						     "text", COL_META_NAME,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, COL_META_NAME);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_reorderable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_META_NAME);
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     COL_META_CONTENT, 
+						     _("Content"),
+						     renderer,
+						     "text", COL_META_CONTENT,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, COL_META_CONTENT);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_reorderable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_META_CONTENT);
+
+
+	/* empty all the DCM fields */
+	page_info_set_text (dialog, properties[PROP_META_DC_TITLE].id, "");
+	page_info_set_text (dialog, properties[PROP_META_DC_DESC].id, "");
+	page_info_set_text (dialog, properties[PROP_META_DC_DATE].id, "");
+	page_info_set_text (dialog, properties[PROP_META_DC_FORMAT].id, "");
+
+	tpage->store = liststore;
+	tpage->selection = selection;
+	tpage->treeview = treeview;
+
+	treeview_info_page_construct (ipage);
+}
+
+static gboolean 
+metadata_add_dc_tag (MetadataInfoPage *page,
+		     EmbedPageMetaTag *tag)
+{
+	InfoPage *ipage = (InfoPage *) page;
+	PageInfoDialog *dialog = ipage->dialog;
+	gboolean added = TRUE;
+
+	if (strcmp (tag->name, "DC.title") == 0)
+	{
+		page_info_set_text (dialog, properties[PROP_META_DC_TITLE].id, tag->content);
+	}
+	else if (strcmp (tag->name, "DC.description") == 0)
+	{
+		page_info_set_text (dialog, properties[PROP_META_DC_DESC].id, tag->content);
+	}
+	else if (strcmp (tag->name, "DC.date") == 0)
+	{
+		page_info_set_text (dialog, properties[PROP_META_DC_DATE].id, tag->content);
+	}
+	else if (strcmp (tag->name, "DC.format") == 0)
+	{
+		page_info_set_text (dialog, properties[PROP_META_DC_FORMAT].id, tag->content);
+	}
+	else
+	{
+		added = FALSE;
+	}
+
+	return added;
+}
+
+static void
+metadata_info_page_fill (InfoPage *ipage)
+{
+	MetadataInfoPage *page = (MetadataInfoPage *) ipage;
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
+	PageInfoDialog *dialog = ipage->dialog;
+	GtkListStore *store = tpage->store;
+	GtkTreeIter iter;
+	GList *tags, *l;
+	GtkWidget *widget;
+	gboolean have_dc = FALSE;
+	char *text;
+
+	tags = dialog->priv->page_info->metatags;
+
+	for (l = tags; l != NULL; l = l->next)
+	{
+		EmbedPageMetaTag *tag = (EmbedPageMetaTag *) l->data;
+
+		if (g_str_has_prefix (tag->name, "DC.") &&
+		    metadata_add_dc_tag (page, tag))
+		{
+			have_dc = TRUE;
+		}
+		else
+		{
+			gtk_list_store_append (store, &iter);
+			gtk_list_store_set (store, &iter,
+					    COL_META_NAME, tag->name,
+					    COL_META_CONTENT, tag->content,
+					    -1);
+		}
+	}
+
+	/* Hide the DC widgets if there's no DC metadata */
+	if (!have_dc)
+	{
+		widget = ephy_dialog_get_control (EPHY_DIALOG (dialog), 
+						  properties[PROP_META_DC_BOX].id);
+		gtk_widget_hide (widget);
+
+		widget = ephy_dialog_get_control (EPHY_DIALOG (dialog), 
+						  properties[PROP_META_OTHER_LABEL].id);
+		text = g_strconcat ("<b>", _("Page Metadata"), "</b>", NULL);
+		gtk_label_set_markup (GTK_LABEL (widget), text);
+		g_free (text);
+	}
+}
+
+static InfoPage *
+metadata_info_page_new (PageInfoDialog *dialog)
+{
+	MetadataInfoPage *page = g_new0 (MetadataInfoPage, 1);
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
+	InfoPage *ipage = (InfoPage *) page;
+
+	ipage->dialog = dialog;
+	ipage->construct = metadata_info_page_construct;
+	ipage->fill = metadata_info_page_fill;
+
+	tpage->popup_path = "/MetadataPopup";
+	tpage->copy_action = "CopyMetadata";
+	tpage->copy_data_col = COL_META_CONTENT;
+	tpage->action_entries = metadata_action_entries;
+	tpage->n_action_entries = G_N_ELEMENTS (metadata_action_entries);
+	tpage->filter_func = treeview_info_page_filter;
 
 	return ipage;
 }
@@ -1789,7 +1904,7 @@ page_info_dialog_init (PageInfoDialog *dialog)
 	dialog->priv->pages[MEDIA_PAGE] = media_info_page_new (dialog);
 	dialog->priv->pages[LINKS_PAGE] = links_info_page_new (dialog);
 	dialog->priv->pages[FORMS_PAGE] = forms_info_page_new (dialog);
-
+	dialog->priv->pages[METADATA_PAGE] = metadata_info_page_new (dialog);
 	/*
 	dialog->priv->pages[SECURITY_PAGE] = security_info_page_new (dialog);
 	*/
@@ -1962,4 +2077,3 @@ page_info_dialog_new (EphyWindow *window,
 			     "embed", embed,
 			     NULL);
 }
-
